package cn.iocoder.yudao.module.wealth.controller.admin.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receivable.ReceivableMapper;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.module.wealth.enums.BoxReportConstant.BOX_AMOUNT_CACHE;

@Component
@Slf4j
@AllArgsConstructor
public class BoxReportCacheLoader implements ApplicationRunner {
    private final ReceivableService receivableService;
    private final ReceivableMapper receivableMapper;
    private final RedisHelper redisHelper;
    @Override
    public void run(ApplicationArguments args) throws Exception {
        log.info("初始化应收报表缓存");
        List<Long> boxIds = receivableMapper.getAllBoxId();
        Set<String> keys = redisHelper.keys(BOX_AMOUNT_CACHE + "*");
        if (CollectionUtil.isNotEmpty(keys)) {
            List<Long> cacheBoxIds = keys.stream().map(key -> Long.parseLong(key.replace(BOX_AMOUNT_CACHE, ""))).collect(Collectors.toList());
            boxIds = boxIds.stream().filter(boxId -> !cacheBoxIds.contains(boxId)).collect(Collectors.toList());
        }
        receivableService.updateBoxAmountCache(boxIds, false);
        log.info("初始化应收报表缓存完成");
    }
}
